------------------------------------------------------------------------------
--									Tooltip support							--
------------------------------------------------------------------------------
local Tooltip =  FlexBar2:NewModule("Tooltip");
-- If this var is incremented, SetDefaults() will be called for this module thus resetting all its settings.
Tooltip.Version = 1;

Tooltip.ButtonMixin = FlexBar2.OO.Mixin 
{
	"ShowTooltip",
	"HideTooltip",
	"EnableTooltip",
	"DisableTooltip",
};


function Tooltip.ButtonMixin:SetDefaults()
	self.InfoTable.ShowTooltip = true;
end

function Tooltip.ButtonMixin:LoadSettings()
	if(self.InfoTable.ShowTooltip == true) then
		self:EnableTooltip();
	else
		self:DisableTooltip();
	end	
end

function Tooltip.ButtonMixin:EnableTooltip()
	self:RegisterScript("OnEnter","ShowTooltip");
	self:RegisterScript("OnLeave","HideTooltip");
end

function Tooltip.ButtonMixin:DisableTooltip()
	self:UnregisterScript("OnEnter","ShowTooltip");
	self:UnregisterScript("OnLeave","HideTooltip");
end

function Tooltip.ButtonMixin:ShowTooltip()
	GameTooltip_SetDefaultAnchor(GameTooltip, self.Frame);
	local ActionType = self:GetModifiedAttribute("type");
	-- Action on button is a spell, call SetSpell on Gametooltip
	if(ActionType == "spell") then
		local SpellSlot = FlexBar2:GetModule("SpellCache"):GetSpellInfo(self:GetModifiedAttribute("spell"));
		if(SpellSlot) then
			GameTooltip:SetSpell(SpellSlot, BOOKTYPE_SPELL);
			GameTooltip:Show();
		end
	-- Action on button is an item, call SetBagItem or SetInventoryItem
	elseif(ActionType == "item") then
		local ItemInfo = FlexBar2:GetModule("ItemCache"):GetItemInfo(self:GetModifiedAttribute("item"));
		if(ItemInfo.Bag and ItemInfo.BagSlot) then
			GameTooltip:SetBagItem(ItemInfo.Bag, ItemInfo.BagSlot);
			GameTooltip:Show();
        elseif(ItemInfo.InvSlot) then
			GameTooltip:SetInventoryItem(ItemInfo.InvSlot);
			GameTooltip:Show();
        else
            FlexBar2:Debug("No tooltip available for item");
        end
	-- Action is a macro, call SetText with the macro's name as arg
	elseif(ActionType == "macro") then
		local Name = GetMacroInfo(self:GetModifiedAttribute("macro"));
		GameTooltip:SetText(Name);
		GameTooltip:Show();
	-- Nothing meaningfull to show, make sure tooltip is hidden
	else
		self:HideTooltip();
	end
end


function Tooltip.ButtonMixin:HideTooltip()
	GameTooltip:Hide()
end

-- GUI -- 
local Waterfall = FlexBar2.Waterfall;
local ShowTooltip;
function Tooltip.ButtonWaterfall(Action, ButtonName, ButtonTable)
	if(Action == "edit") then
		-- 'State button' checkbox --
		ShowTooltip = FlexBar2:GetSetting(ButtonTable, "ShowTooltip");
		Waterfall:AddControl(
		"type","checkbox",
		"text","Show tooltip",
		"width",300,
		"causeRefresh", false,
		"getFunc" ,function() return ShowTooltip; end,
		"setFunc", function(Switch) ShowTooltip = Switch; end);
	elseif(Action == "save") then
		local ButtonList = FlexBar2.Buttons;
		local Button = ButtonList[ButtonTable];
		-- Save wether tooltips are enabled or not
		if(ShowTooltip == true) then
			FlexBar2:ApplySetting(ButtonTable, "ShowTooltip", true);
			Button:EnableTooltip();
		else
			FlexBar2:ApplySetting(ButtonTable, "ShowTooltip", false);
			Button:DisableTooltip();
		end
	end
end

-- SlashCommands --
local SlashCommands = FlexBar2.SlashCommands;
local function SlashCommand_Tooltip(Params)
	local ButtonTable = FlexBar2:ToButtonTable(Params.button);
	local ShowTooltip = Params.tooltip;
	local ButtonList = FlexBar2.Buttons;
	-- Force a state reset after applying the new settings to make sure the button actualy gets it
	if(ShowTooltip == "true") then
		FlexBar2:ApplySetting(ButtonTable, "ShowTooltip", true);
	else
		FlexBar2:ApplySetting(ButtonTable, "ShowTooltip", false);
		
	end
	-- Loop through given button range
	for _, ButtonName in ipairs(ButtonTable) do
		if(ShowTooltip == "true") then
			ButtonList[ButtonName]:EnableTooltip();
		else
			ButtonList[ButtonName]:DisableTooltip();
		end
	end
end

-- Localize this neat verify function for faster access
local VerifyExistance = FlexBar2.SlashCommands.VerifyExistance;
-- AddCommand format :AddCommand(Command, Function, Group, Help)
-- AddParam format :AddParam(Command, Param, Required, AllowedValues, DefaultValue, VerifyFunc) 
-- everything here should be self explainative knowing the syntax
SlashCommands:AddCommand("settooltip", SlashCommand_Tooltip, "tooltip", "/FlexBar2 settooltip tooltip=<true/false>");
SlashCommands:SetAlias("settooltip", "tooltip");
	SlashCommands:AddParam("statebutton", "tooltip", true, {"true", "false"}, nil, nil);
